@@ -195,12 +195,10 @@ rb_debug_init_match (const char *match)
 		"Gtk",
 		"Rhythmbox",
 		"RhythmDB",
-		"MonkeyMedia",
 		"ORBit",
 		"ZVT",
 		"libIDL",
 		"libgconf-scm",
-		"libglade",
 		"libgnomevfs",
 		"librsvg",
 		"GLib-GIO",
