@@ -8,6 +8,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -22,7 +30,7 @@
 #include "config.h"
 #include <glib-object.h>
 #include <glib.h>
-#include <gtk/gtktreeview.h>
+#include <gtk/gtk.h>
 
 #include "rhythmdb.h"
 #include "rhythmdb-query-model.h"
@@ -50,21 +58,23 @@ typedef enum
 GType rhythmdb_property_model_column_get_type (void);
 #define RHYTHMDB_TYPE_PROPERTY_MODEL_COLUMN (rhythmdb_property_model_column_get_type ())
 
+typedef struct _RhythmDBPropertyModel RhythmDBPropertyModel;
+typedef struct _RhythmDBPropertyModelClass RhythmDBPropertyModelClass;
 typedef struct RhythmDBPropertyModelPrivate RhythmDBPropertyModelPrivate;
 
-typedef struct
+struct _RhythmDBPropertyModel
 {
 	GObject parent;
 
 	RhythmDBPropertyModelPrivate *priv;
-} RhythmDBPropertyModel;
+};
 
-typedef struct
+struct _RhythmDBPropertyModelClass
 {
 	GObjectClass parent;
 
 	void (*pre_row_deletion) (RhythmDBPropertyModel *model);
-} RhythmDBPropertyModelClass;
+};
 
 GType			rhythmdb_property_model_get_type	(void);
 
