@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -20,6 +28,15 @@
  *
  */
 
+/**
+ * SECTION:rb-playlist-manager
+ * @short_description: Playlist management object
+ *
+ * The playlist manager loads and saves the on-disk playlist file, provides
+ * UI actions and a DBus interface for dealing with playlists, and internal
+ * interfaces for creating playlists.
+ */
+
 #include "config.h"
 
 #include <string.h>
@@ -29,8 +46,6 @@
 #include <libxml/tree.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libgnomevfs/gnome-vfs.h>
-#include <libgnomevfs/gnome-vfs-mime-utils.h>
 
 #include "rb-playlist-manager.h"
 #include "rb-playlist-source.h"
@@ -48,7 +63,7 @@
 #include "rhythmdb.h"
 #include "rb-stock-icons.h"
 #include "eel-gconf-extensions.h"
-#include "rb-glade-helpers.h"
+#include "rb-builder-helpers.h"
 #include "rb-util.h"
 
 #define RB_PLAYLIST_MGR_VERSION (xmlChar *) "1.0"
@@ -70,6 +85,8 @@ static void rb_playlist_manager_cmd_load_playlist (GtkAction *action,
 						   RBPlaylistManager *mgr);
 static void rb_playlist_manager_cmd_save_playlist (GtkAction *action,
 						   RBPlaylistManager *mgr);
+static void rb_playlist_manager_cmd_save_queue (GtkAction *action,
+						RBPlaylistManager *mgr);
 static void rb_playlist_manager_cmd_new_playlist (GtkAction *action,
 						  RBPlaylistManager *mgr);
 static void rb_playlist_manager_cmd_new_automatic_playlist (GtkAction *action,
@@ -127,6 +144,26 @@ enum
 
 static guint rb_playlist_manager_signals[LAST_SIGNAL] = { 0 };
 
+typedef struct {
+  const gchar *description;
+  /* NULL terminated array of extensions for this file format.  The first
+   * one is the prefered extension for files of this type. */
+  const gchar **extensions;
+  const char *mimetype;
+  const RBPlaylistExportType type;
+} RBPlaylistExportFilter;
+
+static const char *m3u_extensions [] = {"m3u", NULL};
+static const char *pls_extensions [] = {"pls", NULL};
+static const char *xspf_extensions[] = {"xspf", NULL};
+
+static RBPlaylistExportFilter playlist_formats[] = {
+	{N_("MPEG Version 3.0 URL"), m3u_extensions, "audio/x-mpegurl", RB_PLAYLIST_EXPORT_TYPE_M3U},
+	{N_("Shoutcast playlist"), pls_extensions, "audio/x-scpls", RB_PLAYLIST_EXPORT_TYPE_PLS},
+	{N_("XML Shareable Playlist Format"), xspf_extensions, "application/xspf+xml", RB_PLAYLIST_EXPORT_TYPE_XSPF},
+};
+
+
 static GtkActionEntry rb_playlist_manager_actions [] =
 {
 	/* Submenu of Music */
@@ -159,6 +196,9 @@ static GtkActionEntry rb_playlist_manager_actions [] =
 	{ "ShufflePlaylist", NULL, N_("_Shuffle Playlist"), NULL,
 	  N_("Shuffle the tracks in this playlist"),
 	  G_CALLBACK (rb_playlist_manager_cmd_shuffle_playlist) },
+	{ "MusicPlaylistSaveQueue", GTK_STOCK_SAVE_AS, N_("_Save to File..."), NULL,
+	  N_("Save the play queue to a file"),
+	  G_CALLBACK (rb_playlist_manager_cmd_save_queue) },
 };
 static guint rb_playlist_manager_n_actions = G_N_ELEMENTS (rb_playlist_manager_actions);
 
@@ -206,6 +246,14 @@ rb_playlist_manager_class_init (RBPlaylistManagerClass *klass)
 							      "RBSourceList",
 							      RB_TYPE_SOURCELIST,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * RBPlaylistManager::playlist-added:
+	 * @manager: the #RBPlaylistManager
+	 * @source: the new #RBSource
+	 *
+	 * Emitted when a playlist is added, including when being loaded
+	 * from the user's playlist file.
+	 */
 	rb_playlist_manager_signals[PLAYLIST_ADDED] =
 		g_signal_new ("playlist_added",
 			      RB_TYPE_PLAYLIST_MANAGER,
@@ -216,6 +264,13 @@ rb_playlist_manager_class_init (RBPlaylistManagerClass *klass)
 			      G_TYPE_NONE,
 			      1, G_TYPE_OBJECT);
 
+	/**
+	 * RBPlaylistManager::playlist-created:
+	 * @manager: the #RBPlaylistManager
+	 * @source: the newly created playlist #RBSource
+	 *
+	 * Emitted when a new playlist is created.
+	 */
 	rb_playlist_manager_signals[PLAYLIST_CREATED] =
 		g_signal_new ("playlist_created",
 			      RB_TYPE_PLAYLIST_MANAGER,
@@ -226,6 +281,13 @@ rb_playlist_manager_class_init (RBPlaylistManagerClass *klass)
 			      G_TYPE_NONE,
 			      1, G_TYPE_OBJECT);
 
+	/**
+	 * RBPlaylistManager::load-start:
+	 * @manager: the #RBPlaylistManager
+	 *
+	 * Emitted when the playlist manager starts loading the user's
+	 * playlist file.
+	 */
 	rb_playlist_manager_signals[PLAYLIST_LOAD_START] =
 		g_signal_new ("load_start",
 			      RB_TYPE_PLAYLIST_MANAGER,
@@ -235,6 +297,13 @@ rb_playlist_manager_class_init (RBPlaylistManagerClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0, G_TYPE_NONE);
+	/**
+	 * RBPlaylistManager::load-finish
+	 * @manager: the #RBPlaylistManager
+	 *
+	 * Emitted when the playlist manager finishes loading the user's
+	 * playlist file.
+	 */
 	rb_playlist_manager_signals[PLAYLIST_LOAD_FINISH] =
 		g_signal_new ("load_finish",
 			      RB_TYPE_PLAYLIST_MANAGER,
@@ -260,6 +329,13 @@ rb_playlist_manager_init (RBPlaylistManager *mgr)
 	mgr->priv->saving = 0;
 }
 
+/**
+ * rb_playlist_manager_shutdown:
+ * @mgr: the #RBPlaylistManager
+ *
+ * Shuts down the playlist manager, making sure any outstanding playlist save
+ * operation finishes.
+ */
 void
 rb_playlist_manager_shutdown (RBPlaylistManager *mgr)
 {
@@ -491,6 +567,16 @@ rb_playlist_manager_get_property (GObject *object,
 	}
 }
 
+/**
+ * rb_playlist_manager_new:
+ * @shell: the #RBShell
+ * @sourcelist: the #RBSourceList
+ * @playlists_file: the full path to the playlist file to load
+ *
+ * Creates the #RBPlaylistManager instance
+ *
+ * Return value: the #RBPlaylistManager
+ */
 RBPlaylistManager *
 rb_playlist_manager_new (RBShell *shell,
 			 RBSourceList *sourcelist,
@@ -520,12 +606,10 @@ handle_playlist_entry_cb (TotemPlParser *playlist,
 			  RBPlaylistManager *mgr)
 {
 	char *uri;
-#if TOTEM_PL_PARSER_CHECK_VERSION(2,19,0)
 	const char *title, *genre;
 
 	title = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_TITLE);
 	genre = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_GENRE);
-#endif /* TOTEM_PL_PARSER_CHECK_VERSION */
 
 	uri = rb_canonicalise_uri (uri_maybe);
 	g_return_if_fail (uri != NULL);
@@ -566,21 +650,6 @@ playlist_load_started_cb (TotemPlParser *parser, const char *uri, GHashTable *me
 			RB_STATIC_PLAYLIST_SOURCE (rb_playlist_manager_new_playlist (mgr, title, FALSE));
 }
 
-static void
-playlist_load_ended_cb (TotemPlParser *parser, const char *uri, GHashTable *metadata, RBPlaylistManager *mgr)
-{
-	const char *title;
-
-	rb_debug ("finished loading playlist %s", uri);
-
-	title = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_TITLE);
-
-	if (title) {
-		g_object_set (mgr->priv->loading_playlist, "name", title, NULL);
-		mgr->priv->loading_playlist = NULL;
-	}
-}
-
 /**
  * rb_playlist_manager_parse_file:
  * @mgr: the #RBPlaylistManager
@@ -591,7 +660,7 @@ playlist_load_ended_cb (TotemPlParser *parser, const char *uri, GHashTable *meta
  * static playlist.  If the playlist file includes a title, the static
  * playlist created will have the same title.
  *
- * Returns TRUE on success
+ * Return value: TRUE on success
  **/
 gboolean
 rb_playlist_manager_parse_file (RBPlaylistManager *mgr, const char *uri, GError **error)
@@ -611,12 +680,7 @@ rb_playlist_manager_parse_file (RBPlaylistManager *mgr, const char *uri, GError
 					 G_CALLBACK (playlist_load_started_cb),
 					 mgr, 0);
 
-		g_signal_connect_object (parser, "playlist-ended",
-					 G_CALLBACK (playlist_load_ended_cb),
-					 mgr, 0);
-
-		if (g_object_class_find_property (G_OBJECT_GET_CLASS (parser), "recurse"))
-			g_object_set (parser, "recurse", FALSE, NULL);
+		g_object_set (parser, "recurse", FALSE, NULL);
 
 		if (totem_pl_parser_parse (parser, uri, TRUE) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
 			g_set_error (error,
@@ -626,6 +690,7 @@ rb_playlist_manager_parse_file (RBPlaylistManager *mgr, const char *uri, GError
 				     _("The playlist file may be in an unknown format or corrupted."));
 			return FALSE;
 		}
+
 		if (mgr->priv->loading_playlist != NULL) {
 			char *name = NULL;
 
@@ -869,7 +934,7 @@ save_playlist_cb (GtkTreeModel *model,
  * since the last time the playlists were saved, and no save operation is
  * currently taking place.
  *
- * Returns TRUE if a playlist save operation has been started
+ * Return value: TRUE if a playlist save operation has been started
  **/
 gboolean
 rb_playlist_manager_save_playlists (RBPlaylistManager *mgr, gboolean force)
@@ -922,7 +987,7 @@ rb_playlist_manager_save_playlists (RBPlaylistManager *mgr, gboolean force)
  *
  * Creates a new playlist and adds it to the source list.
  *
- * Returns the new playlist object.
+ * Return value: the new playlist object.
  **/
 RBSource *
 rb_playlist_manager_new_playlist (RBPlaylistManager *mgr,
@@ -937,6 +1002,7 @@ rb_playlist_manager_new_playlist (RBPlaylistManager *mgr,
 	else
 		playlist = rb_static_playlist_source_new (mgr->priv->shell,
 							  suggested_name,
+							  NULL,
 							  TRUE,
 							  RHYTHMDB_ENTRY_TYPE_SONG);
 
@@ -1054,7 +1120,7 @@ create_name_from_selection_data (RBPlaylistManager *mgr,
  * Used to implement playlist creation through drag and drop
  * to the source list.
  *
- * Returns the new playlist.
+ * Return value: the new playlist.
  **/
 RBSource *
 rb_playlist_manager_new_playlist_from_selection_data (RBPlaylistManager *mgr,
@@ -1081,7 +1147,7 @@ static void
 rb_playlist_manager_cmd_new_playlist (GtkAction *action,
 				      RBPlaylistManager *mgr)
 {
-	rb_playlist_manager_new_playlist (mgr, NULL, FALSE);
+	rb_playlist_manager_new_playlist (mgr, _("New Playlist"), FALSE);
 }
 
 static void
@@ -1124,7 +1190,7 @@ rb_playlist_manager_cmd_new_automatic_playlist (GtkAction *action,
 		return;
 	}
 
-	playlist = rb_playlist_manager_new_playlist (mgr, NULL, TRUE);
+	playlist = rb_playlist_manager_new_playlist (mgr, _("New Playlist"), TRUE);
 
 	rb_playlist_manager_set_automatic_playlist (mgr, RB_AUTO_PLAYLIST_SOURCE (playlist), creator);
 
@@ -1263,12 +1329,11 @@ static void
 rb_playlist_manager_cmd_shuffle_playlist (GtkAction *action,
 					RBPlaylistManager *mgr)
 {
-	RBStaticPlaylistSource *psource;
+	RhythmDBQueryModel *base_model;
 
-	if (RB_IS_STATIC_PLAYLIST_SOURCE (mgr->priv->selected_source)) {
-		psource = RB_STATIC_PLAYLIST_SOURCE (mgr->priv->selected_source);
-		rb_static_playlist_source_shuffle_playlist (psource);
-	}
+	g_object_get (mgr->priv->selected_source, "base-query-model", &base_model, NULL);
+	rhythmdb_query_model_shuffle_entries (base_model);
+	g_object_unref (base_model);
 }
 
 static void
@@ -1326,36 +1391,37 @@ rb_playlist_manager_cmd_load_playlist (GtkAction *action,
 				       RBPlaylistManager *mgr)
 {
 	GtkWidget *dialog;
+	GtkFileFilter *filter;
+	GtkFileFilter *filter_all;
+	int i;
+
+	filter = gtk_file_filter_new ();
+	gtk_file_filter_set_name (filter, _("Playlists"));
+	for (i = 0; i < G_N_ELEMENTS (playlist_formats); i++) {
+		gtk_file_filter_add_mime_type (filter, playlist_formats[i].mimetype);
+	}
+
+	filter_all = gtk_file_filter_new ();
+	gtk_file_filter_set_name (filter_all, _("All Files"));
+	gtk_file_filter_add_pattern (filter_all, "*");
+
 
 	dialog = rb_file_chooser_new (_("Load Playlist"),
 				      GTK_WINDOW (mgr->priv->window),
 				      GTK_FILE_CHOOSER_ACTION_OPEN,
 				      FALSE);
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter_all);
+	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog), filter);
 
 	g_signal_connect_object (dialog, "response",
 				 G_CALLBACK (load_playlist_response_cb), mgr, 0);
 }
 
-typedef struct {
-  const gchar *description;
-  /* NULL terminated array of extensions for this file format.  The first
-   * one is the prefered extension for files of this type. */
-  const gchar **extensions;
-  const RBPlaylistExportType type;
-} RBPlaylistExportFilter;
-
-static const char *m3u_extensions [] = {"m3u", NULL};
-static const char *pls_extensions [] = {"pls", NULL};
-
-static RBPlaylistExportFilter playlist_export_formats[] = {
-	{N_("MPEG Version 3.0 URL"), m3u_extensions, RB_PLAYLIST_EXPORT_TYPE_M3U},
-	{N_("Shoutcast playlist"), pls_extensions, RB_PLAYLIST_EXPORT_TYPE_PLS},
-};
-
 static void
 save_playlist_response_cb (GtkDialog *dialog,
 			   int response_id,
-			   RBPlaylistManager *mgr)
+			   RBSource *source)
 {
 	char *file = NULL;
 	GtkWidget *menu;
@@ -1378,26 +1444,26 @@ save_playlist_response_cb (GtkDialog *dialog,
 	if (index <= 0) {
 		int i;
 
-		for (i = 0; i < G_N_ELEMENTS (playlist_export_formats); i++) {
+		for (i = 0; i < G_N_ELEMENTS (playlist_formats); i++) {
 			int j;
 
 			/* determine the playlist type from the extension */
-			for (j = 0; playlist_export_formats[i].extensions[j] != NULL; j++) {
-				if (g_str_has_suffix (file, playlist_export_formats[i].extensions[j])) {
-					export_type = playlist_export_formats[i].type;
+			for (j = 0; playlist_formats[i].extensions[j] != NULL; j++) {
+				if (g_str_has_suffix (file, playlist_formats[i].extensions[j])) {
+					export_type = playlist_formats[i].type;
 					break;
 				}
 			}
 		}
 	} else {
-		export_type = playlist_export_formats[index-1].type;
+		export_type = playlist_formats[index-1].type;
 	}
 
 	if (export_type == RB_PLAYLIST_EXPORT_TYPE_UNKNOWN) {
-			rb_error_dialog (NULL, _("Couldn't save playlist"), _("Unsupported file extension given."));
+		rb_error_dialog (NULL, _("Couldn't save playlist"), _("Unsupported file extension given."));
 	} else {
-		rb_playlist_source_save_playlist (RB_PLAYLIST_SOURCE (mgr->priv->selected_source),
-						  file, (export_type == RB_PLAYLIST_EXPORT_TYPE_M3U));
+		rb_playlist_source_save_playlist (RB_PLAYLIST_SOURCE (source),
+						  file, export_type);
 		gtk_widget_destroy (GTK_WIDGET (dialog));
 	}
 
@@ -1418,7 +1484,7 @@ export_set_extension_cb (GtkWidget* widget, GtkDialog *dialog)
 	if (index <= 0)
 		return;
 
-	extension = playlist_export_formats[index-1].extensions[0];
+	extension = playlist_formats[index-1].extensions[0];
 	if (extension == NULL)
 		return;
 
@@ -1472,11 +1538,11 @@ setup_format_menu (GtkWidget* menu, GtkWidget *dialog)
 	gtk_combo_box_set_row_separator_func (GTK_COMBO_BOX (menu), rb_combo_box_hyphen_separator_func,
 					      NULL, NULL);
 
-	for (i = 0; i < G_N_ELEMENTS (playlist_export_formats); i++) {
+	for (i = 0; i < G_N_ELEMENTS (playlist_formats); i++) {
 		gchar *filter_label;
 		GtkTreeIter iter;
 
-		filter_label = filter_get_export_filter_label (&playlist_export_formats[i]);
+		filter_label = filter_get_export_filter_label (&playlist_formats[i]);
 		gtk_list_store_insert_with_values (GTK_LIST_STORE (model), &iter, -1,
 						   0, filter_label, -1);
 
@@ -1489,23 +1555,22 @@ setup_format_menu (GtkWidget* menu, GtkWidget *dialog)
 }
 
 static void
-rb_playlist_manager_cmd_save_playlist (GtkAction *action,
-				       RBPlaylistManager *mgr)
+save_playlist (RBPlaylistManager *mgr, RBSource *source)
 {
-	GladeXML *xml;
-	GtkWidget *dialog, *menu;
-	char *name, *tmp;
+	GtkBuilder *builder;
+	GtkWidget *dialog;
+	GtkWidget *menu;
+	char *name;
+	char *tmp;
 
-	xml = rb_glade_xml_new ("playlist-save.glade",
-				"playlist_save_dialog",
-				mgr);
-	dialog = glade_xml_get_widget (xml, "playlist_save_dialog");
+	builder = rb_builder_load ("playlist-save.ui", mgr);
+	dialog = GTK_WIDGET (gtk_builder_get_object (builder, "playlist_save_dialog"));
 
-	menu = glade_xml_get_widget (xml, "playlist_format_menu");
+	menu = GTK_WIDGET (gtk_builder_get_object (builder, "playlist_format_menu"));
 	setup_format_menu (menu, dialog);
 	g_object_set_data (G_OBJECT (dialog), "export-menu", menu);
 
-	g_object_get (mgr->priv->selected_source, "name", &name, NULL);
+	g_object_get (source, "name", &name, NULL);
 	tmp = g_strconcat (name, ".pls", NULL);
 	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), tmp);
 	g_free (tmp);
@@ -1515,9 +1580,26 @@ rb_playlist_manager_cmd_save_playlist (GtkAction *action,
 	gtk_combo_box_set_active (GTK_COMBO_BOX (menu), 0);
 	g_signal_connect_object (dialog, "response",
 				 G_CALLBACK (save_playlist_response_cb),
-				 mgr, 0);
+				 source, 0);
 
-	g_object_unref (xml);
+	g_object_unref (builder);
+}
+
+static void
+rb_playlist_manager_cmd_save_playlist (GtkAction *action,
+				       RBPlaylistManager *mgr)
+{
+	save_playlist (mgr, mgr->priv->selected_source);
+}
+
+static void
+rb_playlist_manager_cmd_save_queue (GtkAction *action,
+				    RBPlaylistManager *mgr)
+{
+	RBSource *queue;
+	g_object_get (mgr->priv->shell, "queue-source", &queue, NULL);
+	save_playlist (mgr, queue);
+	g_object_unref (queue);
 }
 
 static gboolean
@@ -1553,7 +1635,7 @@ list_playlists_cb (GtkTreeModel *model,
  * rb_playlist_manager_get_playlists
  * @mgr: the #RBPlaylistManager
  *
- * Returns a #GList containing all local playlist source objects.
+ * Return value: a #GList containing all local playlist source objects.
  **/
 GList *
 rb_playlist_manager_get_playlists (RBPlaylistManager *mgr)
@@ -1579,7 +1661,7 @@ rb_playlist_manager_get_playlists (RBPlaylistManager *mgr)
  * Allocates and returns an array containing the names of all local
  * playlists.  This is part of the playlist manager dbus interface.
  *
- * Returns TRUE if successful.
+ * Return value: TRUE if successful.
  **/
 gboolean
 rb_playlist_manager_get_playlist_names (RBPlaylistManager *mgr,
@@ -1670,7 +1752,7 @@ _get_playlist_by_name (RBPlaylistManager *mgr,
  * Will fail if a playlist with that name already exists.
  * This is part of the playlist manager dbus interface.
  *
- * Returns TRUE if successful.
+ * Return value: TRUE if successful.
  **/
 gboolean
 rb_playlist_manager_create_static_playlist (RBPlaylistManager *mgr,
@@ -1699,7 +1781,7 @@ rb_playlist_manager_create_static_playlist (RBPlaylistManager *mgr,
  * Deletes the specified playlist.  Will fail if no playlist with
  * that name exists. This is part of the playlist manager dbus interface.
  *
- * Returns TRUE if successful.
+ * Return value: TRUE if successful.
  */
 gboolean
 rb_playlist_manager_delete_playlist (RBPlaylistManager *mgr,
@@ -1731,7 +1813,7 @@ rb_playlist_manager_delete_playlist (RBPlaylistManager *mgr,
  * Fails if no playlist with that name exists.
  * This is part of the playlist manager dbus interface.
  *
- * Returns TRUE if successful.
+ * Return value: TRUE if successful.
  **/
 gboolean
 rb_playlist_manager_add_to_playlist (RBPlaylistManager *mgr,
@@ -1771,7 +1853,7 @@ rb_playlist_manager_add_to_playlist (RBPlaylistManager *mgr,
  * Fails if no playlist with that name exists.
  * This is part of the playlist manager dbus interface.
  *
- * Returns TRUE if successful.
+ * Return value: TRUE if successful.
  **/
 gboolean
 rb_playlist_manager_remove_from_playlist (RBPlaylistManager *mgr,
@@ -1813,7 +1895,7 @@ rb_playlist_manager_remove_from_playlist (RBPlaylistManager *mgr,
  * Saves the specified playlist to a file in either M3U or PLS format.
  * This is part of the playlist manager dbus interface.
  *
- * Returns TRUE if successful.
+ * Return value: TRUE if successful.
  **/
 gboolean
 rb_playlist_manager_export_playlist (RBPlaylistManager *mgr,
