@@ -7,6 +7,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -51,6 +59,7 @@ GType		rb_static_playlist_source_get_type 	(void);
 
 RBSource *	rb_static_playlist_source_new		(RBShell *shell,
 							 const char *name,
+							 const char *sorting_name,
 							 gboolean local,
 							 RhythmDBEntryType entry_type);
 
@@ -79,6 +88,7 @@ void		rb_static_playlist_source_remove_location(RBStaticPlaylistSource *source,
 void		rb_static_playlist_source_move_entry	(RBStaticPlaylistSource *source,
 							 RhythmDBEntry *entry,
 							 gint index);
+
 G_END_DECLS
 
 #endif /* __RB_STATIC_PLAYLIST_SOURCE_H */
