@@ -1,3 +1,32 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
+ *  Copyright (C) 2007 James Livingston
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2, or (at your option)
+ *  any later version.
+ *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
+ *
+ */
+
+
 #include "config.h"
 
 #include <gtk/gtk.h>
@@ -60,8 +89,8 @@ main (int argc, char **argv)
 	int i;
 
 	if (argc < 2) {
-		g_print ("using ~/.gnome2/rhythmbox/rhythmdb.xml\n");
-		name = g_strdup_printf ("%s/rhythmdb.xml", rb_dot_dir ());
+		name = g_build_filename (rb_user_data_dir(), "rhythmdb.xml", NULL);
+		g_print ("using %s\n", name);
 	} else {
 		name = g_strdup (argv[1]);
 	}
@@ -72,10 +101,9 @@ main (int argc, char **argv)
 	rb_threads_init ();
 	gtk_set_locale ();
 	gtk_init (&argc, &argv);
-	gnome_vfs_init ();
 	rb_debug_init (FALSE);
 	rb_refstring_system_init ();
-	rb_file_helpers_init ();
+	rb_file_helpers_init (TRUE);
 
 	GDK_THREADS_ENTER ();
 
@@ -107,7 +135,6 @@ main (int argc, char **argv)
 	
 	rb_file_helpers_shutdown ();
         rb_refstring_system_shutdown ();
-        gnome_vfs_shutdown ();
 
 
 	rb_profile_end ("load test");
