@@ -1,6 +1,38 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
+ *
+ */
+
+
+#include "config.h"
+
 #include <check.h>
 #include <gtk/gtk.h>
 #include <string.h>
+#include <glib/gi18n.h>
+
+#include "test-utils.h"
 
 #include "rb-debug.h"
 #include "rb-file-helpers.h"
@@ -10,84 +42,14 @@
 #include "rhythmdb-tree.h"
 #include "rhythmdb-query-model.h"
 
-#ifndef fail_if
-#define fail_if(expr, ...) fail_unless(!(expr), "Failure '"#expr"' occured")
-#endif
-
-
-/* test utils */
-gboolean waiting, signaled;
-char *sig_name;
-
-static void
-mark_signal (void)
-{
-	if (signaled) {
-		rb_debug ("got signal '%s' multiple times", sig_name);
-	} else {
-		rb_debug ("got signal '%s'", sig_name);
-		signaled = TRUE;
-		if (waiting)
-			gtk_main_quit ();
-	}
-}
-
-static void
-set_waiting_signal (GObject *o, const char *name)
-{
-	signaled = FALSE;
-	waiting = FALSE;
-	sig_name = g_strdup (name);
-	g_signal_connect (o, sig_name, G_CALLBACK (mark_signal), NULL);
-}
-
-static void wait_for_signal (void)
-{
-	if (!signaled) {
-		rb_debug ("waiting for signal '%s'", sig_name);
-		waiting = TRUE;
-		gtk_main ();
-	} else {
-		rb_debug ("no need to wait for signal '%s', already received", sig_name);
-	}
-
-	g_free (sig_name);
-	sig_name = NULL;
-}
-
-
-/* common setup and teardown */
-RhythmDB *db = NULL;
-
-static void
-test_rhythmdb_setup (void)
-{
-	RhythmDBEntryType entry_type;
-	db = rhythmdb_tree_new ("test");
-	fail_unless (db != NULL, "failed to initialise DB");
-	rhythmdb_start_action_thread (db);
-
-	/* allow SONGs to be synced to for the tests */
-	entry_type = RHYTHMDB_ENTRY_TYPE_SONG;
-	entry_type->can_sync_metadata = (RhythmDBEntryCanSyncFunc)rb_true_function;
-	entry_type->sync_metadata = (RhythmDBEntrySyncFunc)rb_null_function;
-}
-
-static void
-test_rhythmdb_shutdown (void)
-{
-	fail_unless (db != NULL, "failed to shutdown DB");
-	rhythmdb_shutdown (db);
-	g_object_unref (G_OBJECT (db));
-	db = NULL;
-}
-
 static void
 set_true (RhythmDBEntry *entry, gboolean *b)
 {
 	*b = TRUE;
 }
 
+
+
 /* tests */
 START_TEST (test_rhythmdb_indexing)
 {
@@ -95,27 +57,27 @@ START_TEST (test_rhythmdb_indexing)
 	GValue val = {0,};
 	gboolean b;
 
-	entry = rhythmdb_entry_new (db, RHYTHMDB_ENTRY_TYPE_SONG, "file:///whee.ogg");
+	entry = rhythmdb_entry_new (db, RHYTHMDB_ENTRY_TYPE_IGNORE, "file:///whee.ogg");
 	fail_unless (entry != NULL, "failed to create entry");
 
 	g_value_init (&val, G_TYPE_STRING);
 	g_value_set_static_string (&val, "Rock");
-	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_GENRE, &val);
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_GENRE, &val);
 	g_value_unset (&val);
 
 	g_value_init (&val, G_TYPE_STRING);
 	g_value_set_static_string (&val, "Nine Inch Nails");
-	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_ARTIST, &val);
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_ARTIST, &val);
 	g_value_unset (&val);
 
 	g_value_init (&val, G_TYPE_STRING);
 	g_value_set_static_string (&val, "Pretty Hate Machine");
-	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_ALBUM, &val);
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_ALBUM, &val);
 	g_value_unset (&val);
 
 	g_value_init (&val, G_TYPE_STRING);
 	g_value_set_static_string (&val, "Sin");
-	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_TITLE, &val);
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_TITLE, &val);
 	g_value_unset (&val);
 
 	rhythmdb_commit (db);
@@ -183,17 +145,17 @@ START_TEST (test_rhythmdb_multiple)
 	RhythmDBEntry *entry1, *entry2, *entry3;
 
 	/* add multiple entries */
-	entry1 = rhythmdb_entry_new (db, RHYTHMDB_ENTRY_TYPE_SONG, "file:///foo.mp3");
+	entry1 = rhythmdb_entry_new (db, RHYTHMDB_ENTRY_TYPE_IGNORE, "file:///foo.mp3");
 	rhythmdb_commit (db);
 	fail_unless (entry1 != NULL, "failed to create entry");
 	fail_unless (rhythmdb_entry_lookup_by_location (db, "file:///foo.mp3") == entry1, "entry missing");
 
-	entry2 = rhythmdb_entry_new (db, RHYTHMDB_ENTRY_TYPE_SONG, "file:///bar.mp3");
+	entry2 = rhythmdb_entry_new (db, RHYTHMDB_ENTRY_TYPE_IGNORE, "file:///bar.mp3");
 	rhythmdb_commit (db);
 	fail_unless (entry2 != NULL, "failed to create entry");
 	fail_unless (rhythmdb_entry_lookup_by_location (db, "file:///bar.mp3") == entry2, "entry missing");
 
-	entry3 = rhythmdb_entry_new (db, RHYTHMDB_ENTRY_TYPE_SONG, "file:///baz.mp3");
+	entry3 = rhythmdb_entry_new (db, RHYTHMDB_ENTRY_TYPE_IGNORE, "file:///baz.mp3");
 	rhythmdb_commit (db);
 	fail_unless (entry3 != NULL, "failed to create entry");
 	fail_unless (rhythmdb_entry_lookup_by_location (db, "file:///baz.mp3") == entry3, "entry missing");
@@ -228,13 +190,13 @@ START_TEST (test_rhythmdb_mirroring)
 	RhythmDBEntry *entry;
 	const char *str;
 
-	entry = rhythmdb_entry_new (db, RHYTHMDB_ENTRY_TYPE_SONG, "file:///foo.mp3");
+	entry = rhythmdb_entry_new (db, RHYTHMDB_ENTRY_TYPE_IGNORE, "file:///foo.mp3");
 	fail_unless (entry != NULL, "failed to create entry");
 
 	/* check the last-played date is mirrored */
 	g_value_init (&val, G_TYPE_ULONG);
 	g_value_set_ulong (&val, 1354285);
-	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_LAST_PLAYED, &val);
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_LAST_PLAYED, &val);
 	g_value_unset (&val);
 	rhythmdb_commit (db);
 
@@ -268,92 +230,97 @@ START_TEST (test_rhythmdb_mirroring)
 }
 END_TEST
 
-START_TEST (test_rhythmdb_db_queries)
+static int
+count_and_free_refstring_list (GList *list)
 {
-	RhythmDBEntry *entry = NULL;
-	RhythmDBQuery *query;
-	GValue val = {0,};
+	int count;
 
-	entry = rhythmdb_entry_new (db, RHYTHMDB_ENTRY_TYPE_SONG, "file:///whee.ogg");
+	count = g_list_length (list);
+
+	g_list_foreach (list, (GFunc)rb_refstring_unref, NULL);
+	g_list_free (list);
+
+	return count;
+}
+
+START_TEST (test_rhythmdb_keywords)
+{
+	RhythmDBEntry *entry;
+	GList *list;
+	gboolean ret;
+	RBRefString *keyword_foo, *keyword_bar, *keyword_baz;
+
+	keyword_foo = rb_refstring_new ("foo");
+	keyword_bar = rb_refstring_new ("bar");
+	keyword_baz = rb_refstring_new ("baz");
+
+	entry = rhythmdb_entry_new (db, RHYTHMDB_ENTRY_TYPE_IGNORE, "file:///foo.mp3");
 	fail_unless (entry != NULL, "failed to create entry");
 
-	g_value_init (&val, G_TYPE_STRING);
-	g_value_set_static_string (&val, "Rock");
-	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_GENRE, &val);
-	g_value_unset (&val);
+	/* new entries should have 0 keywords */
+	list = rhythmdb_entry_keywords_get (db, entry);
+	fail_unless (count_and_free_refstring_list (list) == 0, "new entry had keywords");
 
-	g_value_init (&val, G_TYPE_STRING);
-	g_value_set_static_string (&val, "Nine Inch Nails");
-	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_ARTIST, &val);
-	g_value_unset (&val);
+	/* adding one keyword */
+	ret = rhythmdb_entry_keyword_add (db, entry, keyword_foo);
+	fail_unless (ret == FALSE, "entry incorrectly reported as having keyword already");
+	list = rhythmdb_entry_keywords_get (db, entry);
+	fail_unless (count_and_free_refstring_list (list) == 1, "entry wrong number of keywords after one was added");
 
-	g_value_init (&val, G_TYPE_STRING);
-	g_value_set_static_string (&val, "Pretty Hate Machine");
-	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_ALBUM, &val);
-	g_value_unset (&val);
+	/* has added keyword */
+	ret = rhythmdb_entry_keyword_has (db, entry, keyword_foo);
+	fail_unless (ret == TRUE, "reported not having just-added keyword");
 
-	g_value_init (&val, G_TYPE_STRING);
-	g_value_set_static_string (&val, "Sin");
-	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_TITLE, &val);
-	g_value_unset (&val);
+	/* add keyword again */
+	ret = rhythmdb_entry_keyword_add (db, entry, keyword_foo);
+	fail_unless (ret == TRUE, "entry incorrectly reported as not keyword already");
 
-	rhythmdb_commit (db);
+	/* check keyword count*/
+	list = rhythmdb_entry_keywords_get (db, entry);
+	fail_unless (count_and_free_refstring_list (list) == 1, "entry wrong number of keywords after one was re-added");
 
-	/* basic queries and conjunctions */
-	query = rhythmdb_query_parse (db,
-				      RHYTHMDB_QUERY_PROP_EQUALS, RHYTHMDB_PROP_TYPE, RHYTHMDB_ENTRY_TYPE_SONG,
-				      RHYTHMDB_QUERY_PROP_EQUALS, RHYTHMDB_PROP_TITLE, "Sin",
-				      RHYTHMDB_QUERY_END);
-	fail_unless (rhythmdb_evaluate_query (db, query, entry), "query evaluated incorrectly");
-	rhythmdb_query_free (query);
-	
-	query = rhythmdb_query_parse (db,
-				      RHYTHMDB_QUERY_PROP_LIKE, RHYTHMDB_PROP_ARTIST, "Nine Inch",
-				      RHYTHMDB_QUERY_END);
-	fail_unless (rhythmdb_evaluate_query (db, query, entry), "query evaluated incorrectly");
-	rhythmdb_query_free (query);
-	
-	query = rhythmdb_query_parse (db,
-				      RHYTHMDB_QUERY_PROP_LIKE, RHYTHMDB_PROP_ALBUM, "Load",
-				      RHYTHMDB_QUERY_END);
-	fail_if (rhythmdb_evaluate_query (db, query, entry), "query evaluated incorrectly");
-	rhythmdb_query_free (query);
-	
-	query = rhythmdb_query_parse (db,
-				      RHYTHMDB_QUERY_PROP_LIKE, RHYTHMDB_PROP_SEARCH_MATCH, "Pretty Nine",
-				      RHYTHMDB_QUERY_END);
-	rhythmdb_query_preprocess (db, query);
-	fail_unless (rhythmdb_evaluate_query (db, query, entry), "query evaluated incorrectly");
-	rhythmdb_query_free (query);
+	/* ensure it has only that keyword */
+	ret = rhythmdb_entry_keyword_has (db, entry, keyword_bar);
+	fail_unless (ret == FALSE, "reported having wrong keyword");
 
-	/* disjunctions */
-	query = rhythmdb_query_parse (db,
-				      RHYTHMDB_QUERY_PROP_LIKE, RHYTHMDB_PROP_TITLE, "Sin",
-				      RHYTHMDB_QUERY_DISJUNCTION,
-				      RHYTHMDB_QUERY_PROP_LIKE, RHYTHMDB_PROP_TITLE, "Son",
-				      RHYTHMDB_QUERY_END);
-	fail_unless (rhythmdb_evaluate_query (db, query, entry), "query evaluated incorrectly");
-	rhythmdb_query_free (query);
-	
-	query = rhythmdb_query_parse (db,
-				      RHYTHMDB_QUERY_PROP_LIKE, RHYTHMDB_PROP_TITLE, "Sun",
-				      RHYTHMDB_QUERY_DISJUNCTION,
-				      RHYTHMDB_QUERY_PROP_LIKE, RHYTHMDB_PROP_TITLE, "Sin",
-				      RHYTHMDB_QUERY_END);
-	fail_unless (rhythmdb_evaluate_query (db, query, entry), "query evaluated incorrectly");
-	rhythmdb_query_free (query);
-	
-	query = rhythmdb_query_parse (db,
-				      RHYTHMDB_QUERY_PROP_LIKE, RHYTHMDB_PROP_TITLE, "Sun",
-				      RHYTHMDB_QUERY_DISJUNCTION,
-				      RHYTHMDB_QUERY_PROP_LIKE, RHYTHMDB_PROP_TITLE, "Son",
-				      RHYTHMDB_QUERY_END);
-	fail_if (rhythmdb_evaluate_query (db, query, entry), "query evaluated incorrectly");
-	rhythmdb_query_free (query);
+	/* remove the keyword */
+	ret = rhythmdb_entry_keyword_remove (db, entry, keyword_foo);
+	fail_unless (ret == TRUE, "reported having not previously having keyword");
 
-	/* TODO: subqueries */
+	/* has removed keyword */
+	ret = rhythmdb_entry_keyword_has (db, entry, keyword_foo);
+	fail_unless (ret == FALSE, "reported having just-removed keyword");
 
-	rhythmdb_entry_delete (db, entry);
+	/* check count is back to zero */
+	list = rhythmdb_entry_keywords_get (db, entry);
+	fail_unless (count_and_free_refstring_list (list) == 0, "entry has keywords after they were removed");
+
+	/* try removing keyword again */
+	ret = rhythmdb_entry_keyword_remove (db, entry, keyword_foo);
+	fail_unless (ret == FALSE, "reported previously having already removed keyword");
+
+	/* add and remove several keywords */
+	ret = rhythmdb_entry_keyword_add (db, entry, keyword_foo);
+	fail_unless (ret == FALSE, "reported previously having already removed keyword");
+	ret = rhythmdb_entry_keyword_add (db, entry, keyword_bar);
+	fail_unless (ret == FALSE, "reported previously having already never-added keyword");
+	ret = rhythmdb_entry_keyword_add (db, entry, keyword_baz);
+	fail_unless (ret == FALSE, "reported previously having already never-added keyword");
+
+	list = rhythmdb_entry_keywords_get (db, entry);
+	fail_unless (count_and_free_refstring_list (list) == 3, "entry wrong number of keywords after several were added");
+	ret = rhythmdb_entry_keyword_remove (db, entry, keyword_foo);
+	fail_unless (ret == TRUE, "reported previously not having added keyword");
+	list = rhythmdb_entry_keywords_get (db, entry);
+	fail_unless (count_and_free_refstring_list (list) == 2, "entry wrong number of keywords after several were added");
+	ret = rhythmdb_entry_keyword_remove (db, entry, keyword_bar);
+	fail_unless (ret == TRUE, "reported previously not having added keyword");
+	list = rhythmdb_entry_keywords_get (db, entry);
+	fail_unless (count_and_free_refstring_list (list) == 1, "entry wrong number of keywords after several were added");
+	ret = rhythmdb_entry_keyword_remove (db, entry, keyword_baz);
+	fail_unless (ret == TRUE, "reported previously not having added keyword");
+	list = rhythmdb_entry_keywords_get (db, entry);
+	fail_unless (count_and_free_refstring_list (list) == 0, "entry wrong number of keywords after several were added");
 }
 END_TEST
 
@@ -373,7 +340,7 @@ START_TEST (test_rhythmdb_deserialisation1)
 	rhythmdb_do_full_query (db, RHYTHMDB_QUERY_RESULTS (model),
 				NULL,
 				RHYTHMDB_QUERY_PROP_EQUALS,
-				RHYTHMDB_PROP_TYPE, RHYTHMDB_ENTRY_TYPE_SONG,
+				RHYTHMDB_PROP_TYPE, RHYTHMDB_ENTRY_TYPE_IGNORE,
 				RHYTHMDB_QUERY_END);
 	wait_for_signal ();
 	fail_unless (gtk_tree_model_iter_n_children (GTK_TREE_MODEL (model), NULL) == 0, "deserialisation incorrect");
@@ -395,9 +362,8 @@ START_TEST (test_rhythmdb_deserialisation2)
 	g_object_set (G_OBJECT (model), "show-hidden", TRUE, NULL);
 	set_waiting_signal (G_OBJECT (model), "complete");
 	rhythmdb_do_full_query (db, RHYTHMDB_QUERY_RESULTS (model),
-				NULL,
 				RHYTHMDB_QUERY_PROP_EQUALS,
-				RHYTHMDB_PROP_TYPE, RHYTHMDB_ENTRY_TYPE_SONG,
+				RHYTHMDB_PROP_TYPE, RHYTHMDB_ENTRY_TYPE_IGNORE,
 				RHYTHMDB_QUERY_END);
 	wait_for_signal ();
 	/* FIXME: this fails for some reason
@@ -424,7 +390,7 @@ START_TEST (test_rhythmdb_deserialisation3)
 	rhythmdb_do_full_query (db, RHYTHMDB_QUERY_RESULTS (model),
 				NULL,
 				RHYTHMDB_QUERY_PROP_EQUALS,
-				RHYTHMDB_PROP_TYPE, RHYTHMDB_ENTRY_TYPE_SONG,
+				RHYTHMDB_PROP_TYPE, RHYTHMDB_ENTRY_TYPE_IGNORE,
 				RHYTHMDB_QUERY_END);
 	wait_for_signal ();
 	/* FIXME: this fails for some reason
@@ -435,33 +401,109 @@ START_TEST (test_rhythmdb_deserialisation3)
 }
 END_TEST
 
+START_TEST (test_rhythmdb_podcast_upgrade)
+{
+	RhythmDBEntry *entry;
+	const char *mountpoint;
+
+	/* load db with old podcasts setups */
+	g_object_set (G_OBJECT (db), "name", SHARE_UNINSTALLED_DIR "/../tests/podcast-upgrade.xml", NULL);
+	set_waiting_signal (G_OBJECT (db), "load-complete");
+	rhythmdb_load (db);
+	wait_for_signal ();
+
+	entry = rhythmdb_entry_lookup_by_location (db, "file:///home/tester/Desktop/BBC%20Xtra/xtra_20080906-1226a.mp3");
+
+	fail_unless (entry != NULL, "entry missing");
+	fail_unless (rhythmdb_entry_get_entry_type (entry) == RHYTHMDB_ENTRY_TYPE_PODCAST_POST, "entry isn't a podcast");
+	mountpoint = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MOUNTPOINT);
+
+	fail_unless (mountpoint != NULL, "no mountpoint for podcast");
+	fail_unless (strcmp (mountpoint, "http://downloads.bbc.co.uk/podcasts/worldservice/xtra/xtra_20080906-1226a.mp3") == 0, "wrong mountpoint for podcast");
+
+	entry = rhythmdb_entry_lookup_by_location (db, "http://downloads.bbc.co.uk/podcasts/worldservice/xtra/xtra_20080903-1217a.mp3");
+	fail_unless (entry != NULL, "entry not upgraded");
+	fail_unless (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MOUNTPOINT) == NULL, "wrong mountpoint for podcast");
+}
+END_TEST
+
+START_TEST (test_rhythmdb_modify_after_delete)
+{
+	RhythmDBEntry *entry;
+	GValue val = {0,};
+
+	entry = rhythmdb_entry_new (db, RHYTHMDB_ENTRY_TYPE_IGNORE, "file:///whee.ogg");
+	fail_unless (entry != NULL, "failed to create entry");
+
+	g_value_init (&val, G_TYPE_STRING);
+	g_value_set_static_string (&val, "Anything");
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_GENRE, &val);
+	g_value_unset (&val);
+	
+	g_value_init (&val, G_TYPE_STRING);
+	g_value_set_static_string (&val, "Nothing");
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_ARTIST, &val);
+	g_value_unset (&val);
+	
+	g_value_init (&val, G_TYPE_STRING);
+	g_value_set_static_string (&val, "Something");
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_ALBUM, &val);
+	g_value_unset (&val);
+	
+	g_value_init (&val, G_TYPE_STRING);
+	g_value_set_static_string (&val, "Thing");
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_TITLE, &val);
+	g_value_unset (&val);
+
+	rhythmdb_commit (db);
+	rhythmdb_entry_ref (entry);
+
+	rhythmdb_entry_delete (db, entry);
+	rhythmdb_commit (db);
+	
+	g_value_init (&val, G_TYPE_STRING);
+	g_value_set_static_string (&val, "Something Else");
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_ALBUM, &val);
+	g_value_unset (&val);
+	
+	rhythmdb_commit (db);
+	rhythmdb_entry_unref (entry);
+}
+END_TEST
 
 static Suite *
 rhythmdb_suite (void)
 {
 	Suite *s = suite_create ("rhythmdb");
-	TCase *tc_chain = tcase_create ("rhythmdb");
+	TCase *tc_chain = tcase_create ("rhythmdb-core");
+	TCase *tc_bugs = tcase_create ("rhythmdb-bugs");
 
 	suite_add_tcase (s, tc_chain);
-	tcase_add_checked_fixture(tc_chain, test_rhythmdb_setup, test_rhythmdb_shutdown);
+	tcase_add_checked_fixture (tc_chain, test_rhythmdb_setup, test_rhythmdb_shutdown);
+	suite_add_tcase (s, tc_bugs);
+	tcase_add_checked_fixture (tc_bugs, test_rhythmdb_setup, test_rhythmdb_shutdown);
+
+	/* test core functionality */
 	/*tcase_add_test (tc_chain, test_refstring);*/
 	tcase_add_test (tc_chain, test_rhythmdb_indexing);
 	tcase_add_test (tc_chain, test_rhythmdb_multiple);
 	tcase_add_test (tc_chain, test_rhythmdb_mirroring);
+	tcase_add_test (tc_chain, test_rhythmdb_keywords);
 	/*tcase_add_test (tc_chain, test_rhythmdb_signals);*/
 	/*tcase_add_test (tc_chain, test_rhythmdb_query);*/
-	tcase_add_test (tc_chain, test_rhythmdb_db_queries);
-	/*tcase_add_test (tc_chain, test_rhythmdb_query_model);*/
-	/*tcase_add_test (tc_chain, test_rhythmdb_chained_query_model);*/
-	/*tcase_add_test (tc_chain, test_rhythmdb_property_model);*/
+	/* FIXME: add some keywords to the deserialisation tests */
 	tcase_add_test (tc_chain, test_rhythmdb_deserialisation1);
 	tcase_add_test (tc_chain, test_rhythmdb_deserialisation2);
 	tcase_add_test (tc_chain, test_rhythmdb_deserialisation3);
 	/*tcase_add_test (tc_chain, test_rhythmdb_serialisation);*/
 
+	/* tests for breakable bug fixes */
+	tcase_add_test (tc_chain, test_rhythmdb_podcast_upgrade);
+	tcase_add_test (tc_chain, test_rhythmdb_modify_after_delete);
+
 	return s;
 }
-	
+
 int
 main (int argc, char **argv)
 {
@@ -469,32 +511,30 @@ main (int argc, char **argv)
 	SRunner *sr;
 	Suite *s;
 
+	g_log_set_always_fatal (G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL);
+
 	/* init stuff */
 	rb_profile_start ("rhythmbox test suite");
 
 	g_thread_init (NULL);
 	rb_threads_init ();
-	gtk_set_locale ();
-	gtk_init (&argc, &argv);
-	gnome_vfs_init ();
 	rb_debug_init (TRUE);
 	rb_refstring_system_init ();
-	rb_file_helpers_init ();
-
-
-	GDK_THREADS_ENTER ();
+	rb_file_helpers_init (TRUE);
 
 	/* setup tests */
 	s = rhythmdb_suite ();
 	sr = srunner_create (s);
+
+	init_setup (sr, argc, argv);
+	init_once (FALSE);
+
 	srunner_run_all (sr, CK_NORMAL);
 	ret = srunner_ntests_failed (sr);
 	srunner_free (sr);
 
-
 	rb_file_helpers_shutdown ();
 	rb_refstring_system_shutdown ();
-	gnome_vfs_shutdown ();
 
 	rb_profile_end ("rhythmbox test suite");
 	return ret;
