@@ -164,12 +164,12 @@ const RBQueryCreatorCriteriaOption numeric_criteria_options[] =
 
 const RBQueryCreatorCriteriaOption year_criteria_options[] =
 {
+	/* Translators: this matches songs within 1-Jan-YEAR to 31-Dec-YEAR */
 	{ N_("in"), 1, RHYTHMDB_QUERY_PROP_YEAR_EQUALS },
-	/* matches if within 1-JAN-YEAR to 31-DEC-YEAR */
+	/* Translators: this matches songs after 31-Dec-YEAR */
 	{ N_("after"), 1, RHYTHMDB_QUERY_PROP_YEAR_GREATER },
-	/* matches if >= 31-DEC-YEAR */
+	/* Translators: this matches songs before 1-Jan-YEAR */
 	{ N_("before"), 1, RHYTHMDB_QUERY_PROP_YEAR_LESS }
-	/* matches if < 1-DEC-YEAR */
 };
 
 const RBQueryCreatorPropertyType rating_property_type =
@@ -367,7 +367,14 @@ integerCriteriaGetWidgetData (GtkWidget *widget, GValue *val)
 static GtkWidget *
 yearCriteriaCreateWidget (gboolean *constrain)
 {
-	return gtk_spin_button_new_with_range (0.0, (double)G_MAXINT, 1.0);
+	GtkWidget *spin;
+	GDate date = {0, };
+	spin = gtk_spin_button_new_with_range (0.0, (double)G_MAXINT, 1.0);
+
+	/* set it to the current year */
+	g_date_set_time_t (&date, time (NULL));
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin), g_date_get_year (&date));
+	return spin;
 }
 
 static void
